use crate::types::project::Project;

pub fn build_html(project: &Project) -> String {
    let users: Vec<String> = project.users.iter().map(|u| u.name.clone()).collect();

    let string = format!(
        "
<!doctype html>
<html lang=\"en\">
  <head>
    <meta charset=\"UTF-8\" />
    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\" />
    <meta name=\"generator\" content=\"msgexport {}\" />
    <title>Chat with {}</title>
    <style>
      body {{
        font-family: Arial, sans-serif;
        background-color: #f9f9f9;
        margin: 0;
        padding: 0;
      }}
      .chat-container {{
        max-width: 800px;
        margin: 20px auto;
        padding: 10px;
        background-color: #ffffff;
        border-radius: 10px;
        box-shadow: 0 4px 10px rgba(0, 0, 0, 0.1);
      }}
      .day-marker {{
        text-align: center;
        margin: 20px 0;
        font-size: 14px;
        color: #888;
      }}
      .message {{
        display: flex;
        flex-direction: column;
        align-items: flex-start;
        margin-bottom: 20px;
      }}
      .message .sender {{
        font-weight: bold;
        margin-bottom: 5px;
        color: #555;
      }}
      .message .content {{
        background-color: #e4f2ff;
        color: #333;
        padding: 10px 15px;
        border-radius: 10px;
        max-width: 70%;
        word-wrap: break-word;
        position: relative;
      }}
      .message .time {{
        font-size: 12px;
        color: #777;
        position: absolute;
        bottom: -18px;
        right: 10px;
        white-space: nowrap;
      }}
      .message img,
      .message video,
      .message audio {{
        display: block;
        margin-top: 5px;
        max-width: 100%;
        border-radius: 5px;
      }}
      .reactions {{
        margin-top: 5px;
        font-size: 12px;
        color: #555;
        display: flex;
        flex-wrap: wrap;
        gap: 5px;
      }}
      .reaction {{
        background-color: #fff;
        color: #333;
        border: 1px solid #ddd;
        border-radius: 15px;
        padding: 5px 10px;
        display: inline-flex;
        align-items: center;
        gap: 5px;
      }}
    </style>
  </head>
  <body>
    <h1 style=\"text-align: center;\">Chat with {}</h1>
    <div class=\"chat-container\" id=\"chat-container\"></div>

    <script type=\"application/json\" id=\"chat-data\">
{}
    </script>
    <script>

      // Fetch the JSON data from the <script> tag
      const data = JSON.parse(document.getElementById(\"chat-data\").textContent);

      // Helper function to format a timestamp to HH:mm
      function formatTime(timestamp) {{
        const date = new Date(timestamp);
        return date.toLocaleTimeString([], {{
          hour: \"2-digit\",
          minute: \"2-digit\",
        }});
      }}

      function formatDate(timestamp) {{
        const date = new Date(timestamp);
        return date.toLocaleDateString([], {{
          weekday: \"long\",
          year: \"numeric\",
          month: \"long\",
          day: \"numeric\",
        }});
      }}

      function createMessageBubble(message, showSenderName) {{
        const div = document.createElement(\"div\");
        div.className = \"message\";

        if (showSenderName) {{
          const senderDiv = document.createElement(\"div\");
          senderDiv.className = \"sender\";
          senderDiv.textContent = message.sender_name;
          div.appendChild(senderDiv);
        }}

        const contentDiv = document.createElement(\"div\");
        contentDiv.className = \"content\";
        contentDiv.setAttribute(\"tooltip\", message.service)

        if (message.text_content) {{
          const text = document.createElement(\"p\");
          text.textContent = message.text_content;
          contentDiv.appendChild(text);
        }}

        if (message.photos.length > 0) {{
          message.photos.forEach((photo) => {{
            const img = document.createElement(\"img\");
            img.src = \"media/\" + photo.img_path;
            contentDiv.appendChild(img);
          }});
        }}

        if (message.videos.length > 0) {{
          message.videos.forEach((video) => {{
            const vid = document.createElement(\"video\");
            vid.src = \"media/\" + video.vid_path;
            vid.controls = true;
            contentDiv.appendChild(vid);
          }});
        }}

        if (message.audio.length > 0) {{
          message.audio.forEach((audio) => {{
            const aud = document.createElement(\"audio\");
            aud.src = \"media/\" +  audio.audio_path;
            aud.controls = true;
            contentDiv.appendChild(aud);
          }});
        }}

        const timeDiv = document.createElement(\"div\");
        timeDiv.className = \"time\";
        timeDiv.textContent = formatTime(message.timestamp);
        contentDiv.appendChild(timeDiv);

        if (message.reactions.length > 0) {{
          const reactionsDiv = document.createElement(\"div\");
          reactionsDiv.className = \"reactions\";
          message.reactions.forEach((reaction) => {{
            const reactionDiv = document.createElement(\"div\");
            reactionDiv.className = \"reaction\";
            reactionDiv.textContent = `${{reaction.actor_name}}: ${{reaction.content}}`;
            reactionsDiv.appendChild(reactionDiv);
          }});
          contentDiv.appendChild(reactionsDiv);
        }}

        div.appendChild(contentDiv);
        return div;
      }}

      function renderChat(data) {{
        const chatContainer = document.getElementById(\"chat-container\");
        let lastSender = null;
        let lastDate = null;

        data.messages.forEach((message) => {{
          const currentDate = formatDate(message.timestamp);

          if (currentDate !== lastDate) {{
            const dayMarker = document.createElement(\"div\");
            dayMarker.className = \"day-marker\";
            dayMarker.textContent = currentDate;
            chatContainer.appendChild(dayMarker);
            lastDate = currentDate;
          }}

          const showSenderName = message.sender_name !== lastSender;
          const bubble = createMessageBubble(message, showSenderName);
          chatContainer.appendChild(bubble);

          lastSender = message.sender_name;
        }});
      }}

      renderChat(data);
    </script>
  </body>
</html>",
        env!("CARGO_PKG_VERSION"),
        users.join(", "),
        users.join(", "),
        serde_json::to_string(project).unwrap()
    );

    string
}
